const isEmpty = require('../validation/isEmpty');
const isNumber = require('../validation/isNumber');

/**
 * Accepts anything and parses into a boolean
 *
 * - Strict mode: will parse 'true', true, 1, and '1' into true, the rest will be false
 * - Lenient mode: will use JS's native boolean parser (!! operator)
 * - falseToUndefined mode: similar to strict mode, but will output undefined instead of false
 *
 * @param {*} s input
 * @param {TypeParser_Options} (options)
 * @returns {Boolean}
 */
module.exports = function (s, options = {}) {

    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'lenient', 'falsetoundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toBoolean`,
            method: 'toBoolean',
        };
    }

    // Lenient mode: behaves like !!
    if (options.mode === 'lenient') {
        return !!s;
    }

    // Already a boolean
    if (typeof s === 'boolean') {
        if (s) {
            return true;
        }
        else {
            return mode === 'strict' ? false : undefined;
        }
    }

    // Null or undefined
    if (isEmpty(s)) {
        return mode === 'strict' ? false : undefined;
    }

    // If it is a number, true = 1
    if (isNumber(s)) {
        if (s === 1) {
            return true;
        }
        else {
            return mode === 'strict' ? false : undefined;
        }
    }

    // Not a string: is false
    if (typeof s !== "string") { return mode === 'strict' ? false : undefined; }

    // It is a string, so let's trim it and check the value
    s = s.trim();

    // If it is a single character, what single characters are considered 'true'?
    if (s.length === 1) {
        if (s === '1') {
            return true;
        }
        else {
            return mode === 'strict' ? false : undefined;
        }
    }

    // Is a string "true" or "false"?
    s = s.toLowerCase();
    if (s === "true")  { return true; }
    if (s === "false") {
        return mode === 'strict' ? false : undefined;
    }

    // default for all other cases
    return mode === 'strict' ? false : undefined;
};