const isObject = require('../validation/isObject');
const isArray = require('../validation/isArray');

/**
 * Receives anything and tries to convert into a number. If it is not possible to be
 * converted into a number (object, array, null, undefined), or it is NaN, it gets
 * converted into 0.
 *
 * If invalid input:
 * - Strict mode: if not possible to convert into a number, it is NaN
 * - NaNto0 mode: similar to strict mode, but will replace NaN with 0
 * - NaNtoUndefined mode: similar to strict mode, but will replace NaN with undefined
 *
 * @param {*} v
 * @param {TypeParser_Options} (options)
 * @returns {Number|undefined}
 */
module.exports = function (v, options = {}) {

    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'nanto0', 'nantoundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toNumber`,
            method: 'toNumber',
        };
    }

    if (// Is null or undefined
        (v === null  || v === undefined) ||

        // Is an object or an array
        (isObject(v) || isArray(v)) ||

        // Is a boolean or an empty string
        (typeof v === 'boolean' || (typeof v === 'string' && v.length < 1)) ||

        // The result or the value is NaN
        (isNaN(+v))
    ) {
        if      (mode === 'strict')         { return NaN;       }
        else if (mode === 'nanto0')         { return 0;         }
        else if (mode === 'nantoundefined') { return undefined; }
    }

    return +v;
};