const isValidTitle = require('./isValidTitle');

const memory = {};

module.exports = {

    /**
     * Creates a custom type
     * @param {{ is: Array, to: Array }} T
     * @param {TypeParser_Type} newType
     */
    register(T, newType) {

        const title    = newType.title;
        const from     = newType.from;
        const isTypeOf = newType.isTypeOf;

        // If a title was not specified...
        if (!title) {
            throw `TypeParser Error: Title not specified for custom type`;
        }

        // If a 'to' method and a 'is' method were not specified...
        if (!from && !isTypeOf) {
            throw `TypeParser Error: You need to specify a method 'from' or/and a method 'isTypeOf' for the type ${title}`;
        }

        // The title must be in upper case and must be a letter
        if (!isValidTitle(title)) {
            throw `TypeParser Error: Invalid title for type (${title}). Titles must start with an upper case letter.`;
        }

        // Registering
        /** @type {TypeParser_Type} */
        const type = {
            from,
            isTypeOf,
            _isTypeParserType: true, // Used to make sure we always receive a valid typeparser type
        };
        memory[title] = type;
        T[title] = {
            from: (value, a1, a2) => {
                return execute(type, 'from', value, a1, a2);
            },
            isTypeOf: (value, a1, a2) => {
                return execute(type, 'isTypeOf', value, a1, a2);
            },
        };
    },


    /**
     * Lists all the methods registered
     *
     * @return {Array<{title: String, hasFrom: Boolean, hasIsTypeOf: Boolean}>}
     */
    list() {
        console.log(`Listing types registered:`);

        const keys = Object.keys(memory);
        const list = [];

        keys.forEach((typeTitle, i) => {
            const hasFrom = !!memory[typeTitle].from;
            const hasIsTypeOf = !!memory[typeTitle].isTypeOf;

            console.log(`${i + 1} - Type \<${typeTitle}\>: can${hasFrom ? '' : ' NOT'} be converted, can${hasIsTypeOf ? '' : ' NOT'} be validated`);

            list.push({
                title: typeTitle,
                hasFrom,
                hasIsTypeOf,
            });
        });

        return list;
    },
};

/**
 * Executes a method (to/is) from the type. If the method is not available, an
 * error is thrown
 * @param {TypeParser_Type} type
 * @param {String} mode
 * @param {*} value
 * @param {Object|Function} (a1)
 * @param {Object|Function} (a2)
 */
function execute(type, mode, value, a1, a2) {

    // Did we get options or callbacks?
    let options = undefined;
    let callback = undefined;
    if (typeof a1 === 'object') {
        options = a1;
        if (a2 && typeof a2 === 'function') {
            callback = a2;
        }
        else if (a2) {
            throwInvalidParameterError();
        }
    }
    else if (typeof a1 === 'function') {
        callback = a1;
        if (a2) {
            throwInvalidParameterError();
        }
    }

    // Did we get a callback, if it is required?
    if (type.requireCallback && !callback) {
        throw `TypeParser Error: A callback for the type ${t.title} is required.`;
    }

    // Did we receive a valid type?
    if (!type._isTypeParserType) {
        let t = type;
        try { t = JSON.parse(type); }
        catch (e) {}
        throw `TypeParser Error: Invalid type received: ${t}. You must use a type that was registered in the TypeParser.`;
    }

    switch(mode) {

        // Conversion
        case 'from':
            if (type.from) {
                return type.from(value, options, callback);
            }
            else {
                throw `TypeParser: Cannot convert to type ${type.title}`;
            }
            break;

        // Validation
        case 'isTypeOf':
            if (type.isTypeOf) {
                return type.isTypeOf(value, options, callback);
            }
            else {
                throw `TypeParser: Cannot check type ${type.title}`;
            }
            break;

        default:
            throw `TypeParser: Invalid mode for execution ${mode}`;
            break;
    }
}

function throwInvalidParameterError() {
    throw `TypeParser Error: Illegal parameters passed to method 'to' or 'is':\n` +
          `  - The first parameter must be a type registered in the type parser\n`+
          `  - The second parameter is optional, an object or a callback\n`+
          `  - The third parameter is optional, or a callback`;
}