const assert = require('assert');

const toArray = require('../../src/conversion/toArray');

const toTestTitle = require('../utils/toTestTitle');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toArray) {
    describe('toArray - { mode: "lenient", forceParseJson: false } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: [0]    },
            { test: -0,   expect: [-0]   },
            { test: 1,    expect: [1]    },
            { test: -1,   expect: [-1]   },
            { test: 0.0,  expect: [0.0]  },
            { test: -0.0, expect: [-0.0] },
            { test: 0.1,  expect: [0.1]  },
            { test: 1.1,  expect: [1.1]  },
            { test: 1.9,  expect: [1.9]  },
            { test: -1.1, expect: [-1.1] },
            { test: -1.9, expect: [-1.9] },

            // Null, NaN and undefined
            { test: null,      expect: [null]      },
            //{ test: NaN,       expect: [NaN]       }, // NaN !== NaN
            { test: undefined, expect: [undefined] },

            // Strings
            { test: 'true',                     expect: ['true']                     },
            { test: 'false',                    expect: ['false']                    },
            { test: 'abc',                      expect: ['abc']                      },
            { test: '',                         expect: ['']                         },
            { test: '0',                        expect: ['0']                        },
            { test: '1',                        expect: ['1']                        },
            { test: '-1',                       expect: ['-1']                       },
            { test: "",                         expect: [""]                         },
            { test: "abc",                      expect: ["abc"]                      },
            { test: "[1,2,3]",                  expect: [1,2,3]                      },
            { test: '{"a":1, "b":3}',           expect: ['{"a":1, "b":3}']           },
            { test: "2017-02-24T19:08:42.835Z", expect: ["2017-02-24T19:08:42.835Z"] },

            // Arrays
            { test: [],      expect: []      },
            { test: [1,2,3], expect: [1,2,3] },
            { test: ["1"],   expect: ["1"]   },

            // Objects
            { test: {a:1}, expect: [{a:1}] },
            { test: {},    expect: [{}]    },

            // Booleans
            { test: false, expect: [false] },
            { test: true,  expect: [true]  },

            // Other
            { test: emptyFunction, expect: [emptyFunction] },
            { test: date,          expect: [date]          },
            { test: invalidDate,   expect: [invalidDate]   },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'lenient' });
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toArray - { mode: "strict", forceParseJson: false }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: [] },
            { test: -0,   expect: [] },
            { test: 1,    expect: [] },
            { test: -1,   expect: [] },
            { test: 0.0,  expect: [] },
            { test: -0.0, expect: [] },
            { test: 0.1,  expect: [] },
            { test: 1.1,  expect: [] },
            { test: 1.9,  expect: [] },
            { test: -1.1, expect: [] },
            { test: -1.9, expect: [] },

            // Null, NaN and undefined
            { test: null,      expect: [] },
            { test: NaN,       expect: [] },
            { test: undefined, expect: [] },

            // Strings
            { test: 'true',                     expect: []      },
            { test: 'false',                    expect: []      },
            { test: 'abc',                      expect: []      },
            { test: '',                         expect: []      },
            { test: '0',                        expect: []      },
            { test: '1',                        expect: []      },
            { test: '-1',                       expect: []      },
            { test: "",                         expect: []      },
            { test: "abc",                      expect: []      },
            { test: "[1,2,3]",                  expect: [1,2,3] },
            { test: '{"a":1, "b":3}',           expect: []      },
            { test: "2017-02-24T19:08:42.835Z", expect: []      },

            // Arrays
            { test: [],      expect:  []      },
            { test: [1,2,3], expect:  [1,2,3] },
            { test: ["1"],   expect:  ["1"]   },

            // Objects
            { test: {a:1}, expect: [] },
            { test: {},    expect: [] },

            // Booleans
            { test: false, expect: [] },
            { test: true,  expect: [] },

            // Other
            { test: emptyFunction, expect: [] },
            { test: date,          expect: [] },
            { test: invalidDate,   expect: [] },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toArray - { mode: "lenient", forceParseJson: true } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: [0]    },
            { test: -0,   expect: [-0]   },
            { test: 1,    expect: [1]    },
            { test: -1,   expect: [-1]   },
            { test: 0.0,  expect: [0.0]  },
            { test: -0.0, expect: [-0.0] },
            { test: 0.1,  expect: [0.1]  },
            { test: 1.1,  expect: [1.1]  },
            { test: 1.9,  expect: [1.9]  },
            { test: -1.1, expect: [-1.1] },
            { test: -1.9, expect: [-1.9] },

            // Null, NaN and undefined
            { test: null,      expect: [null]      },
            //{ test: NaN,       expect: [NaN]       }, // NaN !== NaN
            { test: undefined, expect: [undefined] },

            // Strings
            { test: 'true',                     expect: [true]                       },
            { test: 'false',                    expect: [false]                      },
            { test: 'abc',                      expect: ['abc']                      },
            { test: '',                         expect: ['']                         },
            { test: '0',                        expect: ['0']                        },
            { test: '1',                        expect: ['1']                        },
            { test: '-1',                       expect: ['-1']                       },
            { test: "",                         expect: [""]                         },
            { test: "abc",                      expect: ["abc"]                      },
            { test: "[1,2,3]",                  expect: [1,2,3]                      },
            { test: '{"a":1, "b":3}',           expect: [{a:1, b:3}]                 },
            { test: "2017-02-24T19:08:42.835Z", expect: ["2017-02-24T19:08:42.835Z"] },

            // Arrays
            { test: [],      expect: []      },
            { test: [1,2,3], expect: [1,2,3] },
            { test: ["1"],   expect: ["1"]   },

            // Objects
            { test: {a:1}, expect: [{a:1}] },
            { test: {},    expect: [{}]    },

            // Booleans
            { test: false, expect: [false] },
            { test: true,  expect: [true]  },

            // Other
            { test: emptyFunction, expect: [emptyFunction] },
            { test: date,          expect: [date]          },
            { test: invalidDate,   expect: [invalidDate]   },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'lenient', forceParseJson: true });
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toArray - { mode: "strict", forceParseJson: true }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: [] },
            { test: -0,   expect: [] },
            { test: 1,    expect: [] },
            { test: -1,   expect: [] },
            { test: 0.0,  expect: [] },
            { test: -0.0, expect: [] },
            { test: 0.1,  expect: [] },
            { test: 1.1,  expect: [] },
            { test: 1.9,  expect: [] },
            { test: -1.1, expect: [] },
            { test: -1.9, expect: [] },

            // Null, NaN and undefined
            { test: null,      expect: [] },
            { test: NaN,       expect: [] },
            { test: undefined, expect: [] },

            // Strings
            { test: 'true',                     expect: []      },
            { test: 'false',                    expect: []      },
            { test: 'abc',                      expect: []      },
            { test: '',                         expect: []      },
            { test: '0',                        expect: []      },
            { test: '1',                        expect: []      },
            { test: '-1',                       expect: []      },
            { test: "",                         expect: []      },
            { test: "abc",                      expect: []      },
            { test: "[1,2,3]",                  expect: [1,2,3] },
            { test: '{"a":1, "b":3}',           expect: []      },
            { test: "2017-02-24T19:08:42.835Z", expect: []      },

            // Arrays
            { test: [],      expect:  []      },
            { test: [1,2,3], expect:  [1,2,3] },
            { test: ["1"],   expect:  ["1"]   },

            // Objects
            { test: {a:1}, expect: [] },
            { test: {},    expect: [] },

            // Booleans
            { test: false, expect: [] },
            { test: true,  expect: [] },

            // Other
            { test: emptyFunction, expect: [] },
            { test: date,          expect: [] },
            { test: invalidDate,   expect: [] },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'strict', forceParseJson: true });
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toArray - { mode: "allowUndefined", forceParseJson: true }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: undefined },
            { test: -0,   expect: undefined },
            { test: 1,    expect: undefined },
            { test: -1,   expect: undefined },
            { test: 0.0,  expect: undefined },
            { test: -0.0, expect: undefined },
            { test: 0.1,  expect: undefined },
            { test: 1.1,  expect: undefined },
            { test: 1.9,  expect: undefined },
            { test: -1.1, expect: undefined },
            { test: -1.9, expect: undefined },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: undefined },
            { test: 'false',                    expect: undefined },
            { test: 'abc',                      expect: undefined },
            { test: '',                         expect: undefined },
            { test: '0',                        expect: undefined },
            { test: '1',                        expect: undefined },
            { test: '-1',                       expect: undefined },
            { test: "",                         expect: undefined },
            { test: "abc",                      expect: undefined },
            { test: "[1,2,3]",                  expect: [1,2,3]   },
            { test: '{"a":1, "b":3}',           expect: undefined },
            { test: "2017-02-24T19:08:42.835Z", expect: undefined },

            // Arrays
            { test: [],      expect:  []      },
            { test: [1,2,3], expect:  [1,2,3] },
            { test: ["1"],   expect:  ["1"]   },

            // Objects
            { test: {a:1}, expect: undefined },
            { test: {},    expect: undefined },

            // Booleans
            { test: false, expect: undefined },
            { test: true,  expect: undefined },

            // Other
            { test: emptyFunction, expect: undefined },
            { test: date,          expect: undefined },
            { test: invalidDate,   expect: undefined },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'allowUndefined', forceParseJson: true });
                assert.deepEqual(result, expect);
            });
        });
    });
};