const assert = require('assert');

const toBoolean = require('../../src/conversion/toBoolean');

const toTestTitle = require('../utils/toTestTitle');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toBoolean) {
    describe('toBoolean - { mode: "lenient" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: false },
            { test: -0,   expect: false },
            { test: 1,    expect: true  },
            { test: -1,   expect: true  },
            { test: 0.0,  expect: false },
            { test: -0.0, expect: false },
            { test: 0.1,  expect: true  },
            { test: 1.1,  expect: true  },
            { test: 1.9,  expect: true  },
            { test: -1.1, expect: true  },
            { test: -1.9, expect: true  },

            // Null, NaN and undefined
            { test: null,      expect: false },
            { test: NaN,       expect: false },
            { test: undefined, expect: false },

            // Strings
            { test: 'true',                     expect: true  },
            { test: 'false',                    expect: true  },
            { test: 'abc',                      expect: true  },
            { test: '',                         expect: false },
            { test: '0',                        expect: true  },
            { test: '1',                        expect: true  },
            { test: '-1',                       expect: true  },
            { test: "",                         expect: false },
            { test: "abc",                      expect: true  },
            { test: "[1,2,3]",                  expect: true  },
            { test: '{"a":1, "b":3}',           expect: true  },
            { test: "2017-02-24T19:08:42.835Z", expect: true },

            // Arrays
            { test: [],      expect: true },
            { test: [1,2,3], expect: true },
            { test: ["1"],   expect: true },

            // Objects
            { test: {a:1}, expect: true },
            { test: {},    expect: true },

            // Booleans
            { test: false, expect: false },
            { test: true,  expect: true  },

            // Other
            { test: emptyFunction, expect: true },
            { test: date,          expect: true },
            { test: invalidDate,   expect: true },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'lenient' });
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toBoolean - { mode: "strict" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: false },
            { test: -0,   expect: false },
            { test: 1,    expect: true  },
            { test: -1,   expect: false },
            { test: 0.0,  expect: false },
            { test: -0.0, expect: false },
            { test: 0.1,  expect: false },
            { test: 1.1,  expect: false },
            { test: 1.9,  expect: false },
            { test: -1.1, expect: false },
            { test: -1.9, expect: false },

            // Null, NaN and undefined
            { test: null,      expect: false },
            { test: NaN,       expect: false },
            { test: undefined, expect: false },

            // Strings
            { test: 'true',                     expect: true  },
            { test: 'false',                    expect: false },
            { test: 'abc',                      expect: false },
            { test: '',                         expect: false },
            { test: '0',                        expect: false },
            { test: '1',                        expect: true  },
            { test: '-1',                       expect: false },
            { test: "",                         expect: false },
            { test: "abc",                      expect: false },
            { test: "[1,2,3]",                  expect: false },
            { test: '{"a":1, "b":3}',           expect: false },
            { test: "2017-02-24T19:08:42.835Z", expect: false },

            // Arrays
            { test: [],      expect: false },
            { test: [1,2,3], expect: false },
            { test: ["1"],   expect: false },

            // Objects
            { test: {a:1}, expect: false },
            { test: {},    expect: false },

            // Booleans
            { test: false, expect: false },
            { test: true,  expect: true  },

            // Other
            { test: emptyFunction, expect: false },
            { test: date,          expect: false },
            { test: invalidDate,   expect: false },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toBoolean - { mode: "falseToUndefined" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: undefined },
            { test: -0,   expect: undefined },
            { test: 1,    expect: true      },
            { test: -1,   expect: undefined },
            { test: 0.0,  expect: undefined },
            { test: -0.0, expect: undefined },
            { test: 0.1,  expect: undefined },
            { test: 1.1,  expect: undefined },
            { test: 1.9,  expect: undefined },
            { test: -1.1, expect: undefined },
            { test: -1.9, expect: undefined },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: true      },
            { test: 'false',                    expect: undefined },
            { test: 'abc',                      expect: undefined },
            { test: '',                         expect: undefined },
            { test: '0',                        expect: undefined },
            { test: '1',                        expect: true      },
            { test: '-1',                       expect: undefined },
            { test: "",                         expect: undefined },
            { test: "abc",                      expect: undefined },
            { test: "[1,2,3]",                  expect: undefined },
            { test: '{"a":1, "b":3}',           expect: undefined },
            { test: "2017-02-24T19:08:42.835Z", expect: undefined },

            // Arrays
            { test: [],      expect: undefined },
            { test: [1,2,3], expect: undefined },
            { test: ["1"],   expect: undefined },

            // Objects
            { test: {a:1}, expect: undefined },
            { test: {},    expect: undefined },

            // Booleans
            { test: false, expect: undefined },
            { test: true,  expect: true      },

            // Other
            { test: emptyFunction, expect: undefined },
            { test: date,          expect: undefined },
            { test: invalidDate,   expect: undefined },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'falseToUndefined' });
                assert.deepEqual(result, expect);
            });
        });
    });
};