const assert = require('assert');

const toDate = require('../../src/conversion/toDate');

const toTestTitle = require('../utils/toTestTitle');
const getTimeIfDate = require('../utils/getTimeIfDate');
const encodeNaN = require('../utils/encodeNaN');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toDate) {
    describe('toDate - { mode: "strict" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: 0  },
            { test: -0,   expect: 0  },
            { test: 1,    expect: 1  },
            { test: -1,   expect: -1 },
            { test: 0.0,  expect: 0  },
            { test: -0.0, expect: 0  },
            { test: 0.1,  expect: 0  },
            { test: 1.1,  expect: 1  },
            { test: 1.9,  expect: 1  },
            { test: -1.1, expect: -1 },
            { test: -1.9, expect: -1 },

            // Null, NaN and undefined
            { test: null,      expect: NaN },
            { test: NaN,       expect: NaN },
            { test: undefined, expect: NaN },

            // Strings
            { test: 'true',                     expect: 1             },
            { test: 'false',                    expect: 0             },
            { test: 'abc',                      expect: NaN           },
            { test: '',                         expect: NaN           },
            { test: '0',                        expect: 0             },
            { test: '1',                        expect: 1             },
            { test: '-1',                       expect: -1            },
            { test: "",                         expect: NaN           },
            { test: "abc",                      expect: NaN           },
            { test: "[1,2,3]",                  expect: 1041483600000 },
            { test: '{"a":1, "b":3}',           expect: NaN           },
            { test: "2017-02-24T19:08:42.835Z", expect: 1487963322835 },

            // Arrays
            { test: [],      expect: NaN           },
            { test: [1,2,3], expect: 1041483600000 },
            { test: ["1"],   expect: 1             },

            // Objects
            { test: {a:1}, expect: NaN },
            { test: {},    expect: NaN },

            // Booleans
            { test: false, expect: 0 },
            { test: true,  expect: 1 },

            // Other
            { test: emptyFunction, expect: NaN  },
            { test: date,          expect: date },
            { test: invalidDate,   expect: NaN  },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(getTimeIfDate(encodeNaN(result)), getTimeIfDate(encodeNaN(expect)));
            });
        });
    });

    describe('toDate - { mode: "allowUndefined" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: 0  },
            { test: -0,   expect: 0  },
            { test: 1,    expect: 1  },
            { test: -1,   expect: -1 },
            { test: 0.0,  expect: 0  },
            { test: -0.0, expect: 0  },
            { test: 0.1,  expect: 0  },
            { test: 1.1,  expect: 1  },
            { test: 1.9,  expect: 1  },
            { test: -1.1, expect: -1 },
            { test: -1.9, expect: -1 },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: 1             },
            { test: 'false',                    expect: 0             },
            { test: 'abc',                      expect: undefined     },
            { test: '',                         expect: undefined     },
            { test: '0',                        expect: 0             },
            { test: '1',                        expect: 1             },
            { test: '-1',                       expect: -1            },
            { test: "",                         expect: undefined     },
            { test: "abc",                      expect: undefined     },
            { test: "[1,2,3]",                  expect: 1041483600000 },
            { test: '{"a":1, "b":3}',           expect: undefined     },
            { test: "2017-02-24T19:08:42.835Z", expect: 1487963322835 },

            // Arrays
            { test: [],      expect: undefined     },
            { test: [1,2,3], expect: 1041483600000 },
            { test: ["1"],   expect: 1             },

            // Objects
            { test: {a:1}, expect: undefined },
            { test: {},    expect: undefined },

            // Booleans
            { test: false, expect: 0 },
            { test: true,  expect: 1 },

            // Other
            { test: emptyFunction, expect: undefined   },
            { test: date,          expect: date        },
            { test: invalidDate,   expect: undefined   },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'allowUndefined', forceParseJson: true });
                assert.deepEqual(getTimeIfDate(encodeNaN(result)), getTimeIfDate(encodeNaN(expect)));
            });
        });
    });
};