const assert = require('assert');

const toNumber = require('../../src/conversion/toNumber');

const toTestTitle = require('../utils/toTestTitle');
const encodeNaN = require('../utils/encodeNaN');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toNumber) {
    describe('toNumber - { mode: "NaNto0" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: 0    },
            { test: -0,   expect: -0   },
            { test: 1,    expect: 1    },
            { test: -1,   expect: -1   },
            { test: 0.0,  expect: 0.0  },
            { test: -0.0, expect: -0.0 },
            { test: 0.1,  expect: 0.1  },
            { test: 1.1,  expect: 1.1  },
            { test: 1.9,  expect: 1.9  },
            { test: -1.1, expect: -1.1 },
            { test: -1.9, expect: -1.9 },

            // Null, NaN and undefined
            { test: null,      expect: 0 },
            { test: NaN,       expect: 0 },
            { test: undefined, expect: 0 },

            // Strings
            { test: 'true',                     expect: 0  },
            { test: 'false',                    expect: 0  },
            { test: 'abc',                      expect: 0  },
            { test: '',                         expect: 0  },
            { test: '0',                        expect: 0  },
            { test: '1',                        expect: 1  },
            { test: '-1',                       expect: -1 },
            { test: "",                         expect: 0  },
            { test: "abc",                      expect: 0  },
            { test: "[1,2,3]",                  expect: 0  },
            { test: '{"a":1, "b":3}',           expect: 0  },
            { test: "2017-02-24T19:08:42.835Z", expect: 0  },

            // Arrays
            { test: [],      expect: 0 },
            { test: [1,2,3], expect: 0 },
            { test: ["1"],   expect: 0 },

            // Objects
            { test: {a:1}, expect: 0 },
            { test: {},    expect: 0 },

            // Booleans
            { test: false, expect: 0 },
            { test: true,  expect: 0 },

            // Other
            { test: emptyFunction, expect: 0 },
            { test: date,          expect: 0 },
            { test: invalidDate,   expect: 0 },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'NaNto0' });
                assert.deepEqual(encodeNaN(result), encodeNaN(expect));
            });
        });
    });

    describe('toNumber - { mode: "strict" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: 0    },
            { test: -0,   expect: -0   },
            { test: 1,    expect: 1    },
            { test: -1,   expect: -1   },
            { test: 0.0,  expect: 0.0  },
            { test: -0.0, expect: -0.0 },
            { test: 0.1,  expect: 0.1  },
            { test: 1.1,  expect: 1.1  },
            { test: 1.9,  expect: 1.9  },
            { test: -1.1, expect: -1.1 },
            { test: -1.9, expect: -1.9 },

            // Null, NaN and undefined
            { test: null,      expect: NaN },
            { test: NaN,       expect: NaN },
            { test: undefined, expect: NaN },

            // Strings
            { test: 'true',                     expect: NaN },
            { test: 'false',                    expect: NaN },
            { test: 'abc',                      expect: NaN },
            { test: '',                         expect: NaN },
            { test: '0',                        expect: 0   },
            { test: '1',                        expect: 1   },
            { test: '-1',                       expect: -1  },
            { test: "",                         expect: NaN },
            { test: "abc",                      expect: NaN },
            { test: "[1,2,3]",                  expect: NaN },
            { test: '{"a":1, "b":3}',           expect: NaN },
            { test: "2017-02-24T19:08:42.835Z", expect: NaN },

            // Arrays
            { test: [],      expect: NaN },
            { test: [1,2,3], expect: NaN },
            { test: ["1"],   expect: NaN },

            // Objects
            { test: {a:1}, expect: NaN },
            { test: {},    expect: NaN },

            // Booleans
            { test: false, expect: NaN },
            { test: true,  expect: NaN },

            // Other
            { test: emptyFunction, expect: NaN },
            { test: date,          expect: NaN },
            { test: invalidDate,   expect: NaN },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(encodeNaN(result), encodeNaN(expect));
            });
        });
    });

    describe('toNumber - { mode: "NaNtoUndefined" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: 0    },
            { test: -0,   expect: -0   },
            { test: 1,    expect: 1    },
            { test: -1,   expect: -1   },
            { test: 0.0,  expect: 0.0  },
            { test: -0.0, expect: -0.0 },
            { test: 0.1,  expect: 0.1  },
            { test: 1.1,  expect: 1.1  },
            { test: 1.9,  expect: 1.9  },
            { test: -1.1, expect: -1.1 },
            { test: -1.9, expect: -1.9 },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: undefined },
            { test: 'false',                    expect: undefined },
            { test: 'abc',                      expect: undefined },
            { test: '',                         expect: undefined },
            { test: '0',                        expect: 0         },
            { test: '1',                        expect: 1         },
            { test: '-1',                       expect: -1        },
            { test: "",                         expect: undefined },
            { test: "abc",                      expect: undefined },
            { test: "[1,2,3]",                  expect: undefined },
            { test: '{"a":1, "b":3}',           expect: undefined },
            { test: "2017-02-24T19:08:42.835Z", expect: undefined },

            // Arrays
            { test: [],      expect: undefined },
            { test: [1,2,3], expect: undefined },
            { test: ["1"],   expect: undefined },

            // Objects
            { test: {a:1}, expect: undefined },
            { test: {},    expect: undefined },

            // Booleans
            { test: false, expect: undefined },
            { test: true,  expect: undefined },

            // Other
            { test: emptyFunction, expect: undefined },
            { test: date,          expect: undefined },
            { test: invalidDate,   expect: undefined },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'NaNtoUndefined' });
                assert.deepEqual(encodeNaN(result), encodeNaN(expect));
            });
        });
    });
};


