const assert = require('assert');

const toString = require('../../src/conversion/toString');

const toTestTitle = require('../utils/toTestTitle');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toString) {
    describe('toString - { mode: "lenient" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: '' + 0    },
            { test: -0,   expect: '' + -0   },
            { test: 1,    expect: '' + 1    },
            { test: -1,   expect: '' + -1   },
            { test: 0.0,  expect: '' + 0.0  },
            { test: -0.0, expect: '' + -0.0 },
            { test: 0.1,  expect: '' + 0.1  },
            { test: 1.1,  expect: '' + 1.1  },
            { test: 1.9,  expect: '' + 1.9  },
            { test: -1.1, expect: '' + -1.1 },
            { test: -1.9, expect: '' + -1.9 },

            // Null, NaN and undefined
            { test: null,      expect: '' + null      },
            { test: NaN,       expect: '' + NaN       },
            { test: undefined, expect: '' + undefined },

            // Strings
            { test: 'true',                     expect: 'true'                     },
            { test: 'false',                    expect: 'false'                    },
            { test: 'abc',                      expect: 'abc',                     },
            { test: '',                         expect: '',                        },
            { test: '0',                        expect: '0',                       },
            { test: '1',                        expect: '1',                       },
            { test: '-1',                       expect: '-1',                      },
            { test: "",                         expect: "",                        },
            { test: "abc",                      expect: "abc",                     },
            { test: "[1,2,3]",                  expect: "[1,2,3]",                 },
            { test: '{"a":1, "b":3}',           expect: '{"a":1, "b":3}'           },
            { test: "2017-02-24T19:08:42.835Z", expect: "2017-02-24T19:08:42.835Z" },

            // Arrays
            { test: [],      expect: '' + []      },
            { test: [1,2,3], expect: '' + [1,2,3] },
            { test: ["1"],   expect: '' + ["1"]   },

            // Objects
            { test: {a:1}, expect: '' + {a:1} },
            { test: {},    expect: '' + {}    },

            // Booleans
            { test: false, expect: '' + false },
            { test: true,  expect: '' + true  },

            // Other
            { test: emptyFunction, expect: '' + emptyFunction         },
            { test: date,          expect: '' + date                  },
            { test: invalidDate,   expect: invalidDate.toDateString() },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'lenient' });
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toString - { mode: "strict" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: '0'    },
            { test: -0,   expect: '0'    },
            { test: 1,    expect: '1'    },
            { test: -1,   expect: '-1'   },
            { test: 0.0,  expect: '0'    },
            { test: -0.0, expect: '0'    },
            { test: 0.1,  expect: '0.1'  },
            { test: 1.1,  expect: '1.1'  },
            { test: 1.9,  expect: '1.9'  },
            { test: -1.1, expect: '-1.1' },
            { test: -1.9, expect: '-1.9' },

            // Null, NaN and undefined
            { test: null,      expect: '' },
            { test: NaN,       expect: '' },
            { test: undefined, expect: '' },

            // Strings
            { test: 'true',                     expect: 'true'                     },
            { test: 'false',                    expect: 'false'                    },
            { test: 'abc',                      expect: 'abc',                     },
            { test: '',                         expect: '',                        },
            { test: '0',                        expect: '0',                       },
            { test: '1',                        expect: '1',                       },
            { test: '-1',                       expect: '-1',                      },
            { test: "",                         expect: "",                        },
            { test: "abc",                      expect: "abc",                     },
            { test: "[1,2,3]",                  expect: "[1,2,3]",                 },
            { test: '{"a":1, "b":3}',           expect: '{"a":1, "b":3}'           },
            { test: "2017-02-24T19:08:42.835Z", expect: "2017-02-24T19:08:42.835Z" },

            // Arrays
            { test: [],      expect: '' },
            { test: [1,2,3], expect: '' },
            { test: ["1"],   expect: '' },

            // Objects
            { test: {a:1}, expect: '' },
            { test: {},    expect: '' },

            // Booleans
            { test: false, expect: 'false' },
            { test: true,  expect: 'true'  },

            // Other
            { test: emptyFunction, expect: ''                         },
            { test: date,          expect: date.toDateString()        },
            { test: invalidDate,   expect: invalidDate.toDateString() },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toString - { mode: "emptyAsUndefined" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: '0'    },
            { test: -0,   expect: '0'    },
            { test: 1,    expect: '1'    },
            { test: -1,   expect: '-1'   },
            { test: 0.0,  expect: '0'    },
            { test: -0.0, expect: '0'    },
            { test: 0.1,  expect: '0.1'  },
            { test: 1.1,  expect: '1.1'  },
            { test: 1.9,  expect: '1.9'  },
            { test: -1.1, expect: '-1.1' },
            { test: -1.9, expect: '-1.9' },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: 'true'                     },
            { test: 'false',                    expect: 'false'                    },
            { test: 'abc',                      expect: 'abc',                     },
            { test: '',                         expect: undefined,                 },
            { test: '0',                        expect: '0',                       },
            { test: '1',                        expect: '1',                       },
            { test: '-1',                       expect: '-1',                      },
            { test: "",                         expect: undefined,                 },
            { test: "abc",                      expect: "abc",                     },
            { test: "[1,2,3]",                  expect: "[1,2,3]",                 },
            { test: '{"a":1, "b":3}',           expect: '{"a":1, "b":3}'           },
            { test: "2017-02-24T19:08:42.835Z", expect: "2017-02-24T19:08:42.835Z" },

            // Arrays
            { test: [],      expect: undefined },
            { test: [1,2,3], expect: undefined },
            { test: ["1"],   expect: undefined },

            // Objects
            { test: {a:1}, expect: undefined },
            { test: {},    expect: undefined },

            // Booleans
            { test: false, expect: 'false' },
            { test: true,  expect: 'true'  },

            // Other
            { test: emptyFunction, expect: undefined                  },
            { test: date,          expect: date.toDateString()        },
            { test: invalidDate,   expect: invalidDate.toDateString() },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'emptyAsUndefined' });
                assert.deepEqual(result, expect);
            });
        });
    });
};